@@ -5,7 +5,9 @@ from datetime import timedelta
 from webob import cookies
 from webob.compat import text_
 from webob.compat import native_
-from webob.compat import PY3
+
+py2only = pytest.mark.skipif("sys.version_info >= (3, 0)")
+py3only = pytest.mark.skipif("sys.version_info < (3, 0)")
 
 def setup_module(module):
     cookies._should_raise = True
@@ -38,6 +40,7 @@ def test_cookie_escaped_unquoted():
 def test_cookie_complex():
     c = cookies.Cookie('dismiss-top=6; CP=null*, '
                        'PHPSESSID=0a539d42abc001cdc762809248d4beed, a="42,"')
+
     def d(v):
         return v.decode('ascii')
     c_dict = dict((d(k), d(v.value)) for k, v in c.items())
@@ -72,6 +75,18 @@ def test_cookie_httponly():
     c[b'foo'].httponly = True
     assert c.serialize() == 'foo=bar; HttpOnly'
 
+def test_cookie_samesite_strict():
+    c = cookies.Cookie()
+    c[b"foo"] = b"bar"
+    c[b"foo"].samesite = b"Strict"
+    assert c.serialize() == "foo=bar; SameSite=Strict"
+
+def test_cookie_samesite_lax():
+    c = cookies.Cookie()
+    c[b"foo"] = b"bar"
+    c[b"foo"].samesite = b"Lax"
+    assert c.serialize() == "foo=bar; SameSite=Lax"
+
 def test_cookie_reserved_keys():
     c = cookies.Cookie('dismiss-top=6; CP=null*; $version=42; a=42')
     assert '$version' not in c
@@ -95,6 +110,13 @@ def test_serialize_cookie_date():
     cdate_int = cookies.serialize_cookie_date(10)
     assert cdate_delta == cdate_int
 
+def test_serialize_samesite():
+    assert cookies.serialize_samesite(b"Lax") == b"Lax"
+    assert cookies.serialize_samesite(b"Strict") == b"Strict"
+
+    with pytest.raises(ValueError):
+        cookies.serialize_samesite(b"SomethingElse")
+
 def test_ch_unquote():
     assert cookies._unquote(b'"hello world') == b'"hello world'
     assert cookies._unquote(b'hello world') == b'hello world'
@@ -308,23 +330,45 @@ class TestRequestCookies(object):
         inst = self._makeOne(environ)
         assert sorted(list(inst.items())) == [('a', '1'), ('b', val), ('c', '3')]
 
-    if not PY3:
-        def test_iterkeys(self):
-            environ = {'HTTP_COOKIE': 'a=1; b="La Pe\\303\\261a"; c=3'}
-            inst = self._makeOne(environ)
-            assert sorted(list(inst.iterkeys())) == ['a', 'b', 'c']
+    @py2only
+    def test_iterkeys(self):
+        environ = {'HTTP_COOKIE': 'a=1; b="La Pe\\303\\261a"; c=3'}
+        inst = self._makeOne(environ)
+        assert sorted(list(inst.iterkeys())) == ['a', 'b', 'c']
 
-        def test_itervalues(self):
-            val = text_(b'La Pe\xc3\xb1a', 'utf-8')
-            environ = {'HTTP_COOKIE': 'a=1; b="La Pe\\303\\261a"; c=3'}
-            inst = self._makeOne(environ)
-            sorted(list(inst.itervalues())) == ['1', '3', val]
+    @py3only
+    def test_iterkeys_py3(self):
+        environ = {'HTTP_COOKIE': b'a=1; b="La Pe\\303\\261a"; c=3'.decode('utf-8')}
+        inst = self._makeOne(environ)
+        assert sorted(list(inst.keys())) == ['a', 'b', 'c']
 
-        def test_iteritems(self):
-            val = text_(b'La Pe\xc3\xb1a', 'utf-8')
-            environ = {'HTTP_COOKIE': 'a=1; b="La Pe\\303\\261a"; c=3'}
-            inst = self._makeOne(environ)
-            assert sorted(list(inst.iteritems())) == [('a', '1'), ('b', val), ('c', '3')]
+    @py2only
+    def test_itervalues(self):
+        val = text_(b'La Pe\xc3\xb1a', 'utf-8')
+        environ = {'HTTP_COOKIE': 'a=1; b="La Pe\\303\\261a"; c=3'}
+        inst = self._makeOne(environ)
+        sorted(list(inst.itervalues())) == ['1', '3', val]
+
+    @py3only
+    def test_itervalues_py3(self):
+        val = text_(b'La Pe\xc3\xb1a', 'utf-8')
+        environ = {'HTTP_COOKIE': b'a=1; b="La Pe\\303\\261a"; c=3'.decode('utf-8')}
+        inst = self._makeOne(environ)
+        sorted(list(inst.values())) == ['1', '3', val]
+
+    @py2only
+    def test_iteritems(self):
+        val = text_(b'La Pe\xc3\xb1a', 'utf-8')
+        environ = {'HTTP_COOKIE': 'a=1; b="La Pe\\303\\261a"; c=3'}
+        inst = self._makeOne(environ)
+        assert sorted(list(inst.iteritems())) == [('a', '1'), ('b', val), ('c', '3')]
+
+    @py3only
+    def test_iteritems_py3(self):
+        val = text_(b'La Pe\xc3\xb1a', 'utf-8')
+        environ = {'HTTP_COOKIE': b'a=1; b="La Pe\\303\\261a"; c=3'.decode('utf-8')}
+        inst = self._makeOne(environ)
+        assert sorted(list(inst.items())) == [('a', '1'), ('b', val), ('c', '3')]
 
     def test___contains__(self):
         environ = {'HTTP_COOKIE': 'a=1'}
@@ -616,6 +660,14 @@ class TestSignedCookieProfile(CommonCookieProfile):
         for cookie in ret:
             assert '; HttpOnly' in cookie[1]
 
+    @pytest.mark.parametrize("samesite", [b"Strict", b"Lax"])
+    def test_with_samesite(self, samesite):
+        cookie = self.makeOne(samesite=samesite)
+        ret = cookie.get_headers("test")
+
+        for cookie in ret:
+            assert "; SameSite=" + samesite.decode("ascii") in cookie[1]
+
     def test_cookie_length(self):
         cookie = self.makeOne()
 
